@@ -27,7 +27,7 @@
 #include "libTMCG.hh"
 
 // LibTMCG identifier
-std::string LibTMCG_ID = 
+static const std::string LibTMCG_ID = 
     "LibTMCG " VERSION "  (C) Heiko Stamer, License: GNU GPL version 2";
 
 // LibTMCG general functions
@@ -83,11 +83,43 @@ bool init_libTMCG
 			"] not available" << std::endl;
 		return false;
 	}
-	if (gcry_md_test_algo(GCRY_MD_SHA256)) // check for SHA256
+	if (gcry_md_test_algo(GCRY_MD_SHA256)) // check for SHA2-256
 	{
 		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
 			GCRY_MD_SHA256 << " [" <<
-			"SHA256" <<
+			"SHA2-256" <<
+			"] not available" << std::endl;
+		return false;
+	}
+	if (gcry_md_test_algo(GCRY_MD_SHA384)) // check for SHA2-384
+	{
+		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
+			GCRY_MD_SHA384 << " [" <<
+			"SHA2-384" <<
+			"] not available" << std::endl;
+		return false;
+	}
+	if (gcry_md_test_algo(GCRY_MD_SHA512)) // check for SHA2-512
+	{
+		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
+			GCRY_MD_SHA512 << " [" <<
+			"SHA2-512" <<
+			"] not available" << std::endl;
+		return false;
+	}
+	if (gcry_cipher_test_algo(GCRY_CIPHER_AES)) // check for AES128
+	{
+		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
+			GCRY_CIPHER_AES << " [" <<
+			"AES128" <<
+			"] not available" << std::endl;
+		return false;
+	}
+	if (gcry_cipher_test_algo(GCRY_CIPHER_AES192)) // check for AES192
+	{
+		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
+			GCRY_CIPHER_AES192 << " [" <<
+			"AES192" <<
 			"] not available" << std::endl;
 		return false;
 	}
@@ -102,14 +134,14 @@ bool init_libTMCG
 	return true;
 }
 
-std::string version_libTMCG
+const std::string version_libTMCG
 	()
 {
 	return std::string(VERSION);
 }
 
-std::string identifier_libTMCG
+const std::string identifier_libTMCG
 	()
 {
-	return std::string(LibTMCG_ID);
+	return LibTMCG_ID;
 }
