@@ -159,17 +159,17 @@ void start_instance
 			assert(dkg->CheckGroup());
 			
 			// generating $x$ and extracting $y = g^x \bmod p$
-			std::stringstream err_log, state_log;
+			std::stringstream err_log_generate;
 			bool ret = true;
 			start_clock();
 			std::cout << "P_" << whoami << ": dkg.Generate() at " << time(NULL) << std::endl;
 			if (corrupted)
-				dkg->Generate(aiou, rbc, err_log, true);
+				dkg->Generate(aiou, rbc, err_log_generate, true);
 			else
-				ret = dkg->Generate(aiou, rbc, err_log);
+				ret = dkg->Generate(aiou, rbc, err_log_generate);
 			stop_clock();
 			std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
-			std::cout << "P_" << whoami << ": log follows " << std::endl << err_log.str();
+			std::cout << "P_" << whoami << ": log follows " << std::endl << err_log_generate.str();
 			if (!corrupted)
 				assert(ret);
 			// now: sync for waiting parties
@@ -178,7 +178,27 @@ void start_instance
 			else
 				rbc->Sync(aiounicast::aio_timeout_middle, "step 1");
 
+			// refreshing key shares $x_i$ and $x\prime_i$ for all $P_i$
+			std::stringstream err_log_refresh;
+			start_clock();
+			std::cout << "P_" << whoami << ": dkg.Refresh() at " << time(NULL) << std::endl;
+			if (corrupted)
+				dkg->Refresh(N, whoami, aiou, rbc, err_log_refresh, true);
+			else
+				ret = dkg->Refresh(N, whoami, aiou, rbc, err_log_refresh);
+			stop_clock();
+			std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
+			std::cout << "P_" << whoami << ": log follows " << std::endl << err_log_refresh.str();
+			if (!corrupted)
+				assert(ret);
+			// now: sync for waiting parties
+			if (someone_corrupted)
+				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 2");
+			else
+				rbc->Sync(aiounicast::aio_timeout_middle, "step 2");
+
 			// publish state
+			std::stringstream state_log;
 			std::cout << "P_" << whoami << ": dkg.PublishState() at " << time(NULL) << std::endl;
 			dkg->PublishState(state_log);
 
@@ -204,12 +224,12 @@ void start_instance
 				assert(ret);
 			// now: sync for waiting parties
 			if (someone_corrupted)
-				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 2");
+				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 3");
 			else
-				rbc->Sync(aiounicast::aio_timeout_middle, "step 2");
+				rbc->Sync(aiounicast::aio_timeout_middle, "step 3");
 
 			// signing and verifying messages
-			std::stringstream err_log_sign, err_log_sign_nm1;
+			std::stringstream err_log_sign;
 			mpz_t m, r, s;
 			mpz_init(m), mpz_init(r), mpz_init(s);
 			// check signing and verifying of a message with N signers
@@ -235,6 +255,7 @@ void start_instance
 			std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
 			if (!corrupted)
 				assert(ret);
+			start_clock();
 			std::cout << "P_" << whoami << ": !dss.Verify(43, ...) at " << time(NULL) << std::endl;
 			mpz_add_ui(m, m, 1L);
 			if (corrupted)
@@ -247,12 +268,31 @@ void start_instance
 				assert(!ret);
 			// now: sync for waiting parties
 			if (someone_corrupted)
-				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 3");
+				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 4");
 			else
-				rbc->Sync(aiounicast::aio_timeout_middle, "step 3");
-			// check signing and verifying of a message with N-1 signers = P_1, P_2, ...
+				rbc->Sync(aiounicast::aio_timeout_middle, "step 4");
+			// refreshing key shares $x_i$ and $x\prime_i$ for all $P_i$
+			std::stringstream err_log_refresh_dss;
+			start_clock();
+			std::cout << "P_" << whoami << ": dss.Refresh() at " << time(NULL) << std::endl;
+			if (corrupted)
+				dss->Refresh(N, whoami, aiou, rbc, err_log_refresh_dss, true);
+			else
+				ret = dss->Refresh(N, whoami, aiou, rbc, err_log_refresh_dss);
+			stop_clock();
+			std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
+			std::cout << "P_" << whoami << ": log follows " << std::endl << err_log_refresh_dss.str();
+			if (!corrupted)
+				assert(ret);
+			// now: sync for waiting parties
+			if (someone_corrupted)
+				rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 5");
+			else
+				rbc->Sync(aiounicast::aio_timeout_middle, "step 5");
+			// check signing and verifying of two messages with N-1 signers = P_1, P_2, ... and key share refresh
 			if (whoami > 0)
 			{
+				std::stringstream err_log_sign_nm1;
 				std::map<size_t, size_t> idx2dkg, dkg2idx;
 				for (size_t i = 0; i < (N-1); i++)
 					idx2dkg[i] = i + 1, dkg2idx[i + 1] = i; // create one-to-one mapping
@@ -280,9 +320,56 @@ void start_instance
 					assert(ret);
 				// at the end: sync for waiting parties
 				if (someone_corrupted)
-					rbc_nm1->Sync(aiounicast::aio_timeout_extremely_long, "step 4");
+					rbc_nm1->Sync(aiounicast::aio_timeout_extremely_long, "step 6");
 				else
-					rbc_nm1->Sync(aiounicast::aio_timeout_middle, "step 4");
+					rbc_nm1->Sync(aiounicast::aio_timeout_middle, "step 6");
+				// refreshing key shares $x_i$ and $x\prime_i$ for some $P_i$
+				std::stringstream err_log_refresh_dss2;
+				start_clock();
+				std::cout << "P_" << whoami << ": dss.Refresh() at " << time(NULL) << std::endl;
+				if (corrupted)
+					dss->Refresh(N-1, whoami-1, idx2dkg, dkg2idx, aiou_nm1, rbc_nm1, err_log_refresh_dss2, true);
+				else
+					ret = dss->Refresh(N-1, whoami-1, idx2dkg, dkg2idx, aiou_nm1, rbc_nm1, err_log_refresh_dss2);
+				stop_clock();
+				std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
+				std::cout << "P_" << whoami << ": log follows " << std::endl << err_log_refresh_dss2.str();
+				if (!corrupted)
+					assert(ret);
+				// now: sync for waiting parties
+				if (someone_corrupted)
+					rbc->Sync(aiounicast::aio_timeout_extremely_long, "step 7");
+				else
+					rbc->Sync(aiounicast::aio_timeout_middle, "step 7");
+				// test signing again
+				std::stringstream err_log_sign_nm12;
+				mpz_set_ui(m, 77L), mpz_set_ui(r, 0L), mpz_set_ui(s, 0L);
+				start_clock();
+				std::cout << "P_" << whoami << ": dss.Sign(77, ...) at " << time(NULL) << std::endl;
+				if ((corrupted) && (whoami == (N-1)))
+					dss->Sign(N-1, whoami-1, m, r, s, idx2dkg, dkg2idx, aiou_nm1, rbc_nm1, err_log_sign_nm12, true);
+				else
+					ret = dss->Sign(N-1, whoami-1, m, r, s, idx2dkg, dkg2idx, aiou_nm1, rbc_nm1, err_log_sign_nm12);
+				stop_clock();
+				std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
+				std::cout << "P_" << whoami << ": log follows " << std::endl << err_log_sign_nm12.str();
+				if (!corrupted)
+					assert(ret);
+				start_clock();
+				std::cout << "P_" << whoami << ": dss.Verify(77, ...) at " << time(NULL) << std::endl;
+				if (corrupted)
+					dss->Verify(m, r, s);
+				else
+					ret = dss->Verify(m, r, s);
+				stop_clock();
+				std::cout << "P_" << whoami << ": " << elapsed_time() << std::endl;
+				if (!corrupted)
+					assert(ret);
+				// at the end: sync for waiting parties
+				if (someone_corrupted)
+					rbc_nm1->Sync(aiounicast::aio_timeout_extremely_long, "step 8");
+				else
+					rbc_nm1->Sync(aiounicast::aio_timeout_middle, "step 8");
 			}
 			mpz_clear(m), mpz_clear(r), mpz_clear(s);
 
