@@ -36,7 +36,7 @@ static const char *version = VERSION; // copy VERSION from LibTMCG before overwr
 #include <sys/wait.h>
 #include <signal.h>
 
-#include "dkg-builtin-common.hh"
+#include "dkg-tcpip-common.hh"
 #include "dkg-gnunet-common.hh"
 
 int				pipefd[MAX_N][MAX_N][2], broadcast_pipefd[MAX_N][MAX_N][2];
@@ -50,7 +50,7 @@ int 				opt_verbose = 0;
 char				*opt_crs = NULL;
 char				*opt_passwords = NULL;
 char				*opt_hostname = NULL;
-unsigned long int		opt_t = MAX_N, opt_s = MAX_N, opt_e = 0, opt_p = 35000;
+unsigned long int		opt_t = MAX_N, opt_s = MAX_N, opt_e = 0, opt_p = 55000;
 
 bool				fips = false;
 std::stringstream		crss;
@@ -422,7 +422,8 @@ void run_instance
 		{
 			gcry_mpi_t tmp = gcry_mpi_set_ui(NULL, dss->QUAL[j]);
 			qual.push_back(tmp);
-			capl.push_back(peers[j]);
+			assert(dss->QUAL[j] < peers.size());
+			capl.push_back(peers[dss->QUAL[j]]);
 		}
 		c_ik.resize(dss->n);
 		for (size_t j = 0; j < c_ik.size(); j++)
@@ -449,6 +450,8 @@ void run_instance
 					for (size_t jj = 0; jj < c_ik.size(); jj++)
 						for (size_t kk = 0; kk < c_ik[jj].size(); kk++)
 							gcry_mpi_release(c_ik[jj][kk]);
+					gcry_sexp_release(key);
+					delete dkg, delete dss, delete rbc, delete vtmf, delete aiou, delete aiou2;
 					exit(-1); 
 				}
 				c_ik[j].push_back(tmp);
@@ -1281,7 +1284,9 @@ int main
 	(int argc, char *const *argv)
 {
 	// setup CRS (common reference string) |p| = 2048 bit, |q| = 256 bit
-	// parameters of the underlying group have been generated with dkg-gencrs
+	// The domain parameters of the underlying fixed group have been generated
+	// by the author. You can run dkg-gencrs and use the option "-g" to
+	// employ your own domain parameter set.
 	static std::string crs = "crs|Ya4kG0PQkxSNfXs0GPvNvrLqeyLc13aypAwaHZ3"
 		"kxi6uYagVELARgiqVBVyBe485oVJmXAVATaJIYpQQhgBawHpzPAOyRICy7hQ"
 		"dBvYtmForYx5MGTK7Uf7PPIgnVDQ7csNgsFfMNkUQVKpiszcb4dVi9sGx4Di"
@@ -1532,8 +1537,8 @@ int main
 	if (opt_s != MAX_N)
 		S = opt_s; // get vaule of S from options
 #endif
-	if (T > N)
-		T = N; // apply an upper limit on T
+	if (T >= N)
+		T = (N - 1); // apply an upper limit on T
 	if (S > ((N - 1) / 2))
 		S = (N - 1) / 2; // apply an upper limit on S
 	// check magic of CRS (common reference string)
@@ -1684,18 +1689,18 @@ int main
 	{
 		if (port.length())
 			opt_p = strtoul(port.c_str(), NULL, 10); // get start port from options
-		builtin_init(hostname);
-		builtin_bindports((uint16_t)opt_p, false);
-		builtin_bindports((uint16_t)opt_p, true);
-		while (builtin_connect((uint16_t)opt_p, false) < peers.size())
+		tcpip_init(hostname);
+		tcpip_bindports((uint16_t)opt_p, false);
+		tcpip_bindports((uint16_t)opt_p, true);
+		while (tcpip_connect((uint16_t)opt_p, false) < peers.size())
 			sleep(1);
-		while (builtin_connect((uint16_t)opt_p, true) < peers.size())
+		while (tcpip_connect((uint16_t)opt_p, true) < peers.size())
 			sleep(1);
-		builtin_accept();
-		builtin_fork();
-		ret = builtin_io();
-		builtin_close();
-		builtin_done();
+		tcpip_accept();
+		tcpip_fork();
+		ret = tcpip_io();
+		tcpip_close();
+		tcpip_done();
 		// release cache
 		mpz_ssrandomm_cache_done(cache, cache_mod, &cache_avail);
 		// finish
