@@ -24,8 +24,6 @@
 #endif
 #include "dkg-gnunet-common.hh"
 
-#ifdef FORKING
-
 extern int				pipefd[MAX_N][MAX_N][2], broadcast_pipefd[MAX_N][MAX_N][2];
 extern pid_t				pid[MAX_N];
 extern std::vector<std::string>		peers;
@@ -70,7 +68,6 @@ static bool 					channels_created = false;
 std::string 					thispeer;
 std::map<std::string, size_t> 			peer2pipe;
 std::map<size_t, std::string> 			pipe2peer;
-std::map<size_t, struct GNUNET_PeerIdentity> 	pipe2peerid;
 std::map<size_t, struct GNUNET_CADET_Channel*> 	pipe2channel_out;
 std::map<size_t, struct GNUNET_CADET_Channel*> 	pipe2channel_in;
 
@@ -416,9 +413,6 @@ void gnunet_io(void *cls)
 {
 	io = NULL;
 
-	// NOTE: We need a short pause here, otherwise GNUnet CADET will be disrupted.
-	sleep(1);
-
 	// send messages to peers
 	if (send_queue.size())
 	{
@@ -547,7 +541,7 @@ void gnunet_statistics(void *cls)
 		std::cout << "INFO: send_queue.size() = " << send_queue.size() << ", send_queue_broadcast.size() = " << send_queue_broadcast.size() << std::endl;
 	if (instance_forked)
 	{
-		// shutdown, if forked DKG instance has terminated 
+		// shutdown, if forked instance has terminated 
 		int wstatus = 0;
 		int thispid = pid[peer2pipe[thispeer]];
 		int ret = waitpid(thispid, &wstatus, WNOHANG);
@@ -557,7 +551,7 @@ void gnunet_statistics(void *cls)
 		{
 			if (!WIFEXITED(wstatus))
 			{
-				std::cerr << "ERROR: DKG instance ";
+				std::cerr << "ERROR: protocol instance ";
 				if (WIFSIGNALED(wstatus))
 					std::cerr << thispid << " terminated by signal " << WTERMSIG(wstatus) << std::endl;
 				if (WCOREDUMP(wstatus))
@@ -566,7 +560,7 @@ void gnunet_statistics(void *cls)
 			else if (WIFEXITED(wstatus))
 			{
 				if (gnunet_opt_verbose)
-					std::cerr << "INFO: DKG instance " << thispid << " terminated with exit status " << WEXITSTATUS(wstatus) << std::endl;
+					std::cerr << "INFO: protocol instance " << thispid << " terminated with exit status " << WEXITSTATUS(wstatus) << std::endl;
 			}
 			instance_forked = false;
 			GNUNET_SCHEDULER_shutdown();
@@ -584,7 +578,7 @@ void gnunet_fork(void *cls)
 	{
 		// fork instance
 		if (gnunet_opt_verbose)
-			std::cout << "INFO: forking the DKG instance ..." << std::endl;
+			std::cout << "INFO: forking the protocol instance ..." << std::endl;
 		fork_instance(peer2pipe[thispeer]);
 	}
 	else
@@ -712,6 +706,3 @@ void gnunet_run(void *cls, char *const *args, const char *cfgfile,
 
 #endif
 
-#endif
-
-
